<?php
/**
 * PDO database adapter
 */
class db_pdo extends db_adapter {
    var $db = null;

    /**
     * @param dsn_or_db e.g. "pgsql:dbname=testV24"
     */
    public function db_pdo($dsn_or_db, $host = null, $user = null, $pass = null, $dbase = null) {
        assert('extension_loaded("pdo")');
        $this->table_delimiter = '"';
        if ($dsn_or_db instanceof PDO) {
            // PDO object given => use this
            $this->db = $dsn_or_db;
            return;
        }

        // das / host / user / pass / db given
        assert('is_string($dsn_or_db)');
        assert('is_string($host)');
        assert('is_string($user)');
        assert('is_string($pass)');
        assert('is_string($dbase)');

        $driver_options = array();
        $this->db = new PDO(
            $dsn_or_db.':dbname='.$dbase.';host='.$host,
            $user, $pass, $driver_options
        );
    }

    public function init() {
    }

    public function query($query) {   
        $result = $this->db->exec($query);
        if ($result === false) {
            assert('false /* Invalid query: '.$query.' */');
            die(); // in case that assert does not break
        }
    }

    public function query_row($query) {   
        // TODO faster?
        $all = $this->query_all($query);
        return $all[0];
    }

    public function query_all($query) {
        $all = $this->db->query($query, PDO::FETCH_ASSOC);
        if ($all === false) {
            assert('false /* Invalid query: '.$query.' */');
            die(); // in case that assert does not break
        }
        $result = array();
        foreach ($all as $row) array_push($result, $row);
        return $result;
    }

    /**
     * Optimize this table. Call e.g. from Admin interface or once on long 
     * program run.
     */
    public static function optimize($table) {
        // TODO
    }

    function quote_table_name($table_name) {
        return '"'.$this->escape_string($table_name).'"';
    }

    public function escape_string($str)
    {
        // TODO return mysql_real_escape_string($str);
        return $str;
    }

    public function fetch_columns($table) {
        // FIXME postgres-only
        //$columns = $this->db->query("SHOW COLUMNS FROM " . $table, PDO::FETCH_ASSOC);
        $columns = $this->db->query('SELECT column_name,udt_name FROM information_schema.columns WHERE '.
            'table_name =\''.$table.'\'', PDO::FETCH_ASSOC);
        $result = array();
        foreach ($columns as $key => $col) {
            $result[$col['column_name']] = $this->convert_type($col['udt_name'], $table.'.'.$col['column_name']);
        }
        return $result;
    }

    public function last_insert_id($table) {
        assert('is_string($table)');
        // TODO wie geht das in schoen mit PDO ?
        $query = 'SELECT id FROM '.$this->quote_table_name($table).' ORDER BY id DESC LIMIT 1';
        return $this->select_single($query);
    }

    /**
     *
     */
    public function get_limit($limit) {
        if (is_null($limit)) {
            return '';
        } else if (is_int($limit)) {
            return ' LIMIT '.$limit.' OFFSET 0';
        } else if (is_array($limit)) {
            if (isset($limit['start'])) {
                $start = $limit['start'];
            } else {
                $start = 0;
            }
            assert('is_numeric($start)');
            assert('isset($limit[\'count\'])');
            $count = $limit['count'];
            assert('is_numeric($count)');

            // mysql version
            return " LIMIT $count OFFSET $start";
        }

        die('Wrong $limit parameter');
    }

    private function convert_type($t, $column=null) {
        switch ($t) {
        case 'int2':
        case 'int4':
            return 'integer';
        case 'bool': 
            return 'bool';
        case 'float4':
        case 'float8':
        case 'numeric':
            return 'float';
        case 'date':
            return 'date';
        case 'timestamp': 
            return 'datetime';
        case 'text':
        case 'bpchar':
        case 'varchar':
            return 'string';
        default: 
            die('Which type is: '.$t.' (column: '.$column.')');
            return 'string';
        }
    }

    /**
     * Select one single number from the datase (e.g. a count)
     */
    public function select_single($query) {
        $row = $this->query_row($query);
        assert('sizeof($row) == 1');
        return array_shift($row);
    }
}

